/**
 *	@file	flicker.c
 *	@author	ENDO
 */

typedef	unsigned char	byte;
typedef	unsigned short	hword;
typedef	unsigned int	word;

enum {
	ADDR_REG_BASE       = 0x04000000,
	ADDR_REG_DISPCNT    = ADDR_REG_BASE + 0x0000,
	ADDR_REG_DISPSTAT   = ADDR_REG_BASE + 0x0004,

	ADDR_PALETTE_BG     = 0x05000000,
	ADDR_PALETTE_OBJ    = 0x05000200,

	ADDR_VRAM           = 0x06000000,
	ADDR_VRAM_OBJ       = 0x06010000,

	ADDR_OAM_ATTR       = 0x07000000
};

enum {
	DISPCNT_BGMODE_0 = 0x00U,
	DISPCNT_BGMODE_1 = 0x01U,
	DISPCNT_BGMODE_2 = 0x02U,
	DISPCNT_BGMODE_3 = 0x03U,
	DISPCNT_BGMODE_4 = 0x04U,
	DISPCNT_BGMODE_5 = 0x05U,

	DISPCNT_OBJ_MAPPING_1D = 1U << 6,
	DISPCNT_OBJ_MAPPING_2D = 0U << 6,

	DISPCNT_ENABLE_FORCED_BLANK  = 1U << 7,
	DISPCNT_DISABLE_FORCED_BLANK = 0U << 7,

	DISPCNT_ENABLE_BG0    = 1U <<  8,
	DISPCNT_ENABLE_BG1    = 1U <<  9,
	DISPCNT_ENABLE_BG2    = 1U << 10,
	DISPCNT_ENABLE_BG3    = 1U << 11,
	DISPCNT_ENABLE_OBJ    = 1U << 12,

	DISPCNT_ENABLE_WIN0   = 1U << 13,
	DISPCNT_ENABLE_WIN1   = 1U << 14,
	DISPCNT_ENABLE_WINOBJ = 1U << 15
};

enum {
	DISPSTAT_VBLANK_STATUS_OUTSIDE = 0U << 0,
	DISPSTAT_VBLANK_STATUS_INSIDE  = 1U << 0,

	DISPSTAT_HBLANK_STATUS_OUTSIDE = 0U << 1,
	DISPSTAT_HBLANK_STATUS_INSIDE  = 1U << 1,

	DISPSTAT_VCOUNTER_NON_MATCH = 0U << 2,
	DISPSTAT_VCOUNTER_MATCH     = 1U << 2,

	DISPSTAT_VBLANK_INTERRUPT_ENABLE  = 1U << 3,
	DISPSTAT_VBLANK_INTERRUPT_DISABLE = 0U << 3,

	DISPSTAT_HBLANK_INTERRUPT_ENABLE  = 1U << 4,
	DISPSTAT_HBLANK_INTERRUPT_DISABLE = 0U << 4,

	DISPSTAT_VCOUNTER_MATCH_INTERRUPT_ENABLE  = 1U << 5,
	DISPSTAT_VCOUNTER_MATCH_INTERRUPT_DISABLE = 0U << 5
};

static void WaitForVBlank(void)
{
	while ((*(volatile hword *)ADDR_REG_DISPSTAT & DISPSTAT_VBLANK_STATUS_INSIDE) != 0) {
		;	/*EMPTY*/
	}
	while ((*(volatile hword *)ADDR_REG_DISPSTAT & DISPSTAT_VBLANK_STATUS_INSIDE) == 0) {
		;	/*EMPTY*/
	}
}

int main(void)
{
	int counter = 0;

	*(volatile hword *)ADDR_REG_DISPCNT = (hword)(DISPCNT_BGMODE_0 | DISPCNT_DISABLE_FORCED_BLANK);

	for (;;) {
		static const hword table_palettes[] = {0xf800U, 0x07e0U, 0x001fU};

		*(volatile hword *)ADDR_PALETTE_BG = table_palettes[counter % 3];
		counter++;

		WaitForVBlank();
	}

	/*NOTREACHED*/
	return 0;
}
